"use server";

import { revalidatePath } from "next/cache";
import { db } from "../db";
import { regions } from "../db/schema";
import { ZodError, z } from "zod";
import { eq } from "drizzle-orm";

export const addRegion = async (prevstate: any, formData: FormData) => {
  //assign formdaata to variables.
  const name = (formData.get("name") as string).toLowerCase();
  const countryId = formData.get("country") as string;

  //check if region already exists in country
  const exists = await db
    .select({ name: regions.name })
    .from(regions)
    .where(eq(regions.countryId, countryId) && eq(regions.name, name));

  //Define the schema for the form data
  const schema = z.object({
    countryId: z.string().min(1, "No country selected"),
    name: z
      .string()
      .min(1, "Name is required")
      .refine(() => !exists, {
        message: `${name} already exists in selected country`,
      }),
  });

  //Parse the form data using the schema for validation, and check if the name already exists
  try {
    schema.parse({
      countryId,
      name,
    });
    //If the name doesn't exist, add the country to the database abd revalidate the page
    await db.insert(regions).values({ countryId, name });
    revalidatePath("/");
    //Return a success message
    return {
      message: "success",
      errors: undefined,
      fieldValues: {
        name: "",
        countryId: "",
      },
    };
  } catch (error) {
    const zodError = error as ZodError;
    const errorMap = zodError.flatten().fieldErrors;
    //Return an error object with the field values and errors.
    return {
      message: "error",
      errors: {
        name: errorMap["name"]?.[0] ?? "",
        countryId: errorMap["countryId"]?.[0] ?? "",
      },
      fieldValues: {
        name,
        countryId,
      },
    };
  }
};
